from __future__ import annotations

from os import environ

from hcloud import Client
from hcloud.images import Image
from hcloud.server_types import ServerType
from hcloud.servers import Server
from hcloud.volumes import Volume

assert (
    "HCLOUD_TOKEN" in environ
), "Please export your API token in the HCLOUD_TOKEN environment variable"
token = environ["HCLOUD_TOKEN"]

client = Client(token=token)

# Create 2 servers
response1 = client.servers.create(
    name="Server1", server_type=ServerType(name="cx22"), image=Image(id=4711)
)

response2 = client.servers.create(
    "Server2", server_type=ServerType(name="cx22"), image=Image(id=4711)
)

server1 = response1.server
server2 = response2.server

# Get all servers

servers = client.servers.get_all()

assert servers[0].id == server1.id
assert servers[1].id == server2.id

# Create 2 volumes

response1 = client.volumes.create(size=15, name="Volume1", location=server1.location)
response2 = client.volumes.create(size=10, name="Volume2", location=server2.location)

volume1 = response1.volume
volume2 = response2.volume

# Attach volume to server

client.volumes.attach(server1, volume1)
client.volumes.attach(server2, volume2)

# Detach second volume

client.volumes.detach(volume2)

# Poweroff 2nd server
client.servers.power_off(server2)

# Create one more volume and attach it to server with id=33

server33 = Server(id=33)
response = client.volumes.create(size=33, name="Volume33", server=server33)

print(response.action.status)

# Create one more server and attach 2 volumes to it
client.servers.create(
    "Server3",
    server_type=ServerType(name="cx22"),
    image=Image(id=4711),
    volumes=[Volume(id=221), Volume(id=222)],
)



from __future__ import annotations

from os import environ

from hcloud import Client

assert (
    "HCLOUD_TOKEN" in environ
), "Please export your API token in the HCLOUD_TOKEN environment variable"
token = environ["HCLOUD_TOKEN"]

client = Client(token=token)
servers = client.servers.get_all()
print(servers)



from __future__ import annotations

import json
from datetime import datetime, timedelta
from os import environ

from hcloud import Client
from hcloud.images import Image
from hcloud.server_types import ServerType

assert (
    "HCLOUD_TOKEN" in environ
), "Please export your API token in the HCLOUD_TOKEN environment variable"
token = environ["HCLOUD_TOKEN"]

client = Client(token=token)

server = client.servers.get_by_name("my-server")
if server is None:
    response = client.servers.create(
        name="my-server",
        server_type=ServerType(name="cx22"),
        image=Image(name="ubuntu-24.04"),
    )
    server = response.server

end = datetime.now()
start = end - timedelta(hours=1)

response = server.get_metrics(
    type=["cpu", "network"],
    start=start,
    end=end,
)

print(json.dumps(response.metrics))



from __future__ import annotations


class BaseDomain:
    __api_properties__: tuple

    @classmethod
    def from_dict(cls, data: dict):  # type: ignore[no-untyped-def]
        """
        Build the domain object from the data dict.
        """
        supported_data = {k: v for k, v in data.items() if k in cls.__api_properties__}
        return cls(**supported_data)

    def __repr__(self) -> str:
        kwargs = [f"{key}={getattr(self, key)!r}" for key in self.__api_properties__]  # type: ignore[var-annotated]
        return f"{self.__class__.__qualname__}({', '.join(kwargs)})"


class DomainIdentityMixin:

    id: int | None
    name: str | None

    @property
    def id_or_name(self) -> int | str:
        """
        Return the first defined value, and fails if none is defined.
        """
        if self.id is not None:
            return self.id
        if self.name is not None:
            return self.name
        raise ValueError("id or name must be set")

    def has_id_or_name(self, id_or_name: int | str) -> bool:
        """
        Return whether this domain has the same id or same name as the other.

        The domain calling this method MUST be a bound domain or be populated, otherwise
        the comparison will not work as expected (e.g. the domains are the same but
        cannot be equal, if one provides an id and the other the name).
        """
        values: list[int | str] = []
        if self.id is not None:
            values.append(self.id)
        if self.name is not None:
            values.append(self.name)
        if not values:
            raise ValueError("id or name must be set")

        return id_or_name in values


class Pagination(BaseDomain):
    __api_properties__ = (
        "page",
        "per_page",
        "previous_page",
        "next_page",
        "last_page",
        "total_entries",
    )
    __slots__ = __api_properties__

    def __init__(
        self,
        page: int,
        per_page: int,
        previous_page: int | None = None,
        next_page: int | None = None,
        last_page: int | None = None,
        total_entries: int | None = None,
    ):
        self.page = page
        self.per_page = per_page
        self.previous_page = previous_page
        self.next_page = next_page
        self.last_page = last_page
        self.total_entries = total_entries


class Meta(BaseDomain):
    __api_properties__ = ("pagination",)
    __slots__ = __api_properties__

    def __init__(self, pagination: Pagination | None = None):
        self.pagination = pagination

    @classmethod
    def parse_meta(cls, response: dict) -> Meta | None:
        """
        If present, extract the meta details from the response and return a meta object.
        """
        meta = None
        if response and "meta" in response:
            meta = cls()
            try:
                meta.pagination = Pagination(**response["meta"]["pagination"])
            except KeyError:
                pass

        return meta



from __future__ import annotations

from os import environ

from hcloud import Client
from hcloud.images import Image
from hcloud.server_types import ServerType

assert (
    "HCLOUD_TOKEN" in environ
), "Please export your API token in the HCLOUD_TOKEN environment variable"
token = environ["HCLOUD_TOKEN"]

# Create a client
client = Client(token=token)

# Create 2 servers
# Create 2 servers
response1 = client.servers.create(
    "Server1", server_type=ServerType(name="cx22"), image=Image(id=4711)
)

response2 = client.servers.create(
    "Server2", server_type=ServerType(name="cx22"), image=Image(id=4711)
)
# Get all servers
server1 = response1.server
server2 = response2.server

servers = client.servers.get_all()

assert servers[0].id == server1.id
assert servers[1].id == server2.id
# Create 2 volumes

response1 = client.volumes.create(size=15, name="Volume1", location=server1.location)
response2 = client.volumes.create(size=10, name="Volume2", location=server2.location)

volume1 = response1.volume
volume2 = response2.volume

# Attach volume to server

volume1.attach(server1)
volume2.attach(server2)

# Detach second volume

volume2.detach()

# Poweroff 2nd server
server2.power_off()



from __future__ import annotations

from os import environ

from hcloud import Client
from hcloud.images import Image
from hcloud.server_types import ServerType

assert (
    "HCLOUD_TOKEN" in environ
), "Please export your API token in the HCLOUD_TOKEN environment variable"
token = environ["HCLOUD_TOKEN"]

client = Client(token=token)

response = client.servers.create(
    name="my-server",
    server_type=ServerType(name="cx22"),
    image=Image(name="ubuntu-24.04"),
)
server = response.server
print(server)
print("Root Password" + response.root_password)
